/*
 * QEstEidCommon
 *
 * Copyright (C) 2009-2012 Jargo Kõster <jargo@innovaatik.ee>
 * Copyright (C) 2009-2012 Raul Metsma <raul@innovaatik.ee>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 */

#include "Diagnostics.h"

#include "Common.h"
#include "QPCSC.h"

#include <QtCore/QProcess>
#include <QtCore/QSettings>
#include <QtCore/QTextStream>

#include <qt_windows.h>

static QString getLibVersion( const QString &lib )
{
	DWORD infoHandle;
	LONG sz = GetFileVersionInfoSize( LPCWSTR(lib.utf16()), &infoHandle );
	if( !sz )
		return QString( "%1 (%2)" ).arg( lib, Diagnostics::tr("failed to get version info") );

	QByteArray data( sz * 2, 0 );
	if( !GetFileVersionInfoW( LPCWSTR(lib.utf16()), 0, sz, data.data() ) )
		return QString( "%1 (%2)" ).arg( lib, Diagnostics::tr("failed to get version info") );

	VS_FIXEDFILEINFO *info;
	UINT len;
	if( !VerQueryValueW( data.constData(), L"\\", (LPVOID*)&info, &len ) )
		return QString( "%1 (%2)" ).arg( lib, Diagnostics::tr("failed to get version info") );

	return QString( "%1 (%2.%3.%4.%5)" ).arg( lib )
		.arg( HIWORD(info->dwFileVersionMS) )
		.arg( LOWORD(info->dwFileVersionMS) )
		.arg( HIWORD(info->dwFileVersionLS) )
		.arg( LOWORD(info->dwFileVersionLS) );
}

static QString getUserRights()
{
	HANDLE hToken = NULL;
	if ( !OpenThreadToken( GetCurrentThread(), TOKEN_QUERY, TRUE, &hToken ) )
	{
		if ( GetLastError() != ERROR_NO_TOKEN )
			return Diagnostics::tr( "Unknown - error %1" ).arg( GetLastError() );
		if ( !OpenProcessToken( GetCurrentProcess(), TOKEN_QUERY, &hToken ) )
			return Diagnostics::tr( "Unknown - error %1" ).arg( GetLastError() );
	}

	DWORD dwLength = 0;
	if ( !GetTokenInformation( hToken, TokenGroups, NULL, dwLength, &dwLength ) )
	{
		if( GetLastError() != ERROR_INSUFFICIENT_BUFFER )
			return Diagnostics::tr( "Unknown - error %1" ).arg( GetLastError() );
	}

	PTOKEN_GROUPS pGroup = (PTOKEN_GROUPS)GlobalAlloc( GPTR, dwLength );
	if ( !GetTokenInformation( hToken, TokenGroups, pGroup, dwLength, &dwLength ) )
	{
		if ( pGroup )
			GlobalFree( pGroup );
		return Diagnostics::tr( "Unknown - error %1" ).arg( GetLastError() );;
	}

	QString rights = Diagnostics::tr( "User" );
	SID_IDENTIFIER_AUTHORITY NtAuthority = SECURITY_NT_AUTHORITY;
	PSID AdministratorsGroup = NULL;
	if( AllocateAndInitializeSid( &NtAuthority, 2, SECURITY_BUILTIN_DOMAIN_RID,
			DOMAIN_ALIAS_RID_ADMINS, 0, 0, 0, 0, 0, 0, &AdministratorsGroup ) )
	{
		for ( DWORD dwIndex = 0; dwIndex < pGroup->GroupCount; dwIndex++ )
		{
			if ( EqualSid( AdministratorsGroup, pGroup->Groups[dwIndex].Sid ) )
			{
				rights = Diagnostics::tr( "Administrator" );
				break;
			}
		}
	}

	if ( AdministratorsGroup )
		FreeSid( AdministratorsGroup );
	if ( pGroup )
		GlobalFree( pGroup );

	return rights;
}

void Diagnostics::run()
{
	QString info;
	QTextStream s( &info );

	s << "<b>" << tr("Locale (time-, number format / codepage):") << "</b> ";
	QLocale::Language language = QLocale::system().language();
	QString locale = (language == QLocale::C ? "English/United States" : QLocale::languageToString( language ) );
	CPINFOEX CPInfoEx;
	if( GetCPInfoEx( GetConsoleCP(), 0, &CPInfoEx ) != 0 )
		locale.append( " / " ).append( QString( (QChar*)CPInfoEx.CodePageName ) );
	s << locale << "<br />";
	emit update( info );
	info.clear();

	s << "<b>" << tr("User rights: ") << "</b>" << getUserRights() << "<br />";
	emit update( info );
	info.clear();

	QStringList base = Common::packages( QStringList() << "Eesti ID-kaardi tarkvara", false );
	if( !base.isEmpty() )
		s << "<b>" << tr("Base version:") << "</b> " << base.join( "<br />" ) << "<br />";
	s << "<b>" << tr("Application version:") << "</b> "<< QCoreApplication::applicationVersion() << "<br />";
	emit update( info );
	info.clear();

	s << "<b>" << tr("OS:") << "</b> " << Common::applicationOs() << "<br />";
	SYSTEM_INFO sysinfo;
	GetSystemInfo( &sysinfo );
	s << "<b>" << tr("CPU:") << "</b> " << QString::number( sysinfo.dwProcessorType ) << "<br /><br />";
	emit update( info );
	info.clear();

	s << "<b>" << tr("Arguments:") << "</b> " << qApp->arguments().join(" ") << "<br />";
	s << "<b>" << tr("Library paths:") << "</b> " << QCoreApplication::libraryPaths().join( ";" ) << "<br />";

	s << "<b>" << tr("Libraries") << ":</b><br />";
	s << getLibVersion( "digidoc" ) << "<br />";
	s << getLibVersion( "digidocpp" ) << "<br />";
	s << getLibVersion( "qdigidocclient.exe" ) << "<br />";
	s << getLibVersion( "qdigidoccrypto.exe" ) << "<br />";
	s << getLibVersion( "qesteidutil.exe" ) << "<br />";
	s << getLibVersion( "advapi32" ) << "<br />";
	s << getLibVersion( "crypt32" ) << "<br />";
	s << getLibVersion( "winscard" ) << "<br />";
	s << getLibVersion( "esteidcsp" ) << "<br />";
	s << getLibVersion( "esteidcm" ) << "<br />";
	s << getLibVersion( "libeay32" ) << "<br />";
	s << getLibVersion( "ssleay32" ) << "<br />";
	s << getLibVersion( "opensc-pkcs11" ) << "<br />";
	s << "QT (" << qVersion() << ")<br />" << "<br />";
	emit update( info );
	info.clear();

	if( QSysInfo::windowsVersion() >= QSysInfo::WV_VISTA )
	{
		enum {
			Running,
			Stopped,
			NotFound
		} result = NotFound;
		if( SC_HANDLE h = OpenSCManager( NULL, NULL, SC_MANAGER_CONNECT ) )
		{
			if( SC_HANDLE s = OpenService( h, L"atrfiltr", SERVICE_QUERY_STATUS ) )
			{
				SERVICE_STATUS status;
				QueryServiceStatus( s, &status );
				result = (status.dwCurrentState == SERVICE_RUNNING) ? Running : Stopped;
				CloseServiceHandle( s );
			}
			CloseServiceHandle( h );
		}
		s << "<br /><b>" << tr("ATRfiltr service status: ") << "</b>" << " ";
		switch( result )
		{
		case NotFound: s << tr("Not found"); break;
		case Stopped: s << tr("Not running"); break;
		case Running: s << tr("Running"); break;
		}
	}
	s << "<br />";

	s << getReaderInfo() << "<br /><br />";
	emit update( info );
	info.clear();

	QStringList browsers = Common::packages( QStringList() << "Mozilla" << "Google Chrome" );
	QSettings reg( "HKEY_LOCAL_MACHINE\\SOFTWARE\\Microsoft\\Internet Explorer", QSettings::NativeFormat );
	browsers << QString( "Internet Explorer (%1)" ).arg(
		reg.value("svcVersion", reg.value( "Version" ) ).toString() );
	s << "<b>" << tr("Browsers:") << "</b><br />" << browsers.join( "<br />" ) << "<br /><br />";
	emit update( info );
	info.clear();

	QProcess p;
	p.start( "opensc-tool", QStringList() << "-la" );
	p.waitForFinished();
	QString cmd = QString::fromUtf8( p.readAll() );
	if( !cmd.isEmpty() )
		s << "<b>" << tr("OpenSC tool:") << "</b><br/> " << cmd.replace( "\n", "<br />" ) << "<br />";
	emit update( info );
	info.clear();

	p.start( "pkcs11-tool", QStringList() << "-T" << QString("--module=%1").arg( PKCS11_MODULE ) );
	p.waitForFinished();
	cmd = QString::fromUtf8( p.readAll() );
	if( !cmd.isEmpty() )
		s << "<b>" << tr("PKCS11 tool:") << "</b><br/> " << cmd.replace( "\n", "<br />" ) << "<br />";
	emit update( info );
	info.clear();

	QApplication::restoreOverrideCursor();
}
